import 'dart:io';

import 'package:camera/camera.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:uuid/uuid.dart';
import 'package:video_player/video_player.dart';
import 'package:xcommonwidgets/xcommonwidgets.dart';
import 'package:xfxbase/XFDataItem.dart';
import 'package:xfxwidgets/XF_MAIN/XApp.dart';
import 'package:xfxwidgets/xMediaItem.dart';
import 'package:xfxwidgets/xfxwidgets.dart';
import 'package:xtoolspackage/XLibStorage.dart';
import 'package:xtoolspackage/xtoolspackage.dart';
import 'package:audio_waveforms/audio_waveforms.dart';
import 'package:video_thumbnail/video_thumbnail.dart' as thumb;

class XMediaPlayer<T> extends StatefulWidget {
  //Ci facciamo passare l'oggetto da visualizzare T
  XMediaPlayer(
    Key? key, {
    required this.itemOriginal,
    required this.onTake_Picture,
    required this.onTake_Video,
    required this.onTake_Audio,
    required this.xApp,
    this.title,
    this.title_BackColor,
    this.title_Color,
  });

  String? title;
  Color? title_BackColor;
  Color? title_Color;

  T itemOriginal;
  Function(XFile, T, bool, void Function(void Function())) onTake_Picture;
  Function(XFile, T, bool, void Function(void Function())) onTake_Video;
  Function(String, T, bool, void Function(void Function())) onTake_Audio;
  XApp xApp;
  @override
  State<XMediaPlayer<T>> createState() => XMediaPlayerState<T>();
}

class XMediaPlayerState<T> extends State<XMediaPlayer<T>> with WidgetsBindingObserver, TickerProviderStateMixin {
  late CameraController? photoController;
  int pointers = 0;
  static String? urlRTSP;
  XFile? imageFile;
  XFile? videoFile;
  VideoPlayerController? videoController;
  VoidCallback? videoPlayerListener;
  double _minAvailableZoom = 1.0;
  double _maxAvailableZoom = 1.0;
  double _currentScale = 1.0;
  double _baseScale = 1.0;
  // ignore: unused_field
  double _minAvailableExposureOffset = 0.0;
  // ignore: unused_field
  double _maxAvailableExposureOffset = 0.0;
  bool outCamera = false;

  bool isBusy = false;
  String waiterMessage = "";

  TextEditingController? urlRTSPcontroller = TextEditingController();

  bool? createAudio = false;
  File? xFileUI;
  List<XMediaItem> lXMediaUI = List<XMediaItem>.empty(growable: true);

  late final RecorderController recorderController;

  String? musicFile;
  bool isRecording_Audio = false;
  bool isRecordingCompleted = false;
  bool isLoading = true;
  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    XStatefulWidgetBase.color_Waiter = Color.fromARGB(255, 22, 228, 255);
    loadPlayer();
    if (urlRTSP == null) {
      urlRTSPcontroller!.text = "rtsp://";
    } else {
      urlRTSPcontroller!.text = urlRTSP ?? "";
    }
    super.initState();
    _initialiseControllers();
  }

  loadPlayer() async {
    await onNewCameraSelected();
    setFlashMode(FlashMode.off, setState);
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    photoController!.dispose();
    urlRTSPcontroller!.dispose();
    // player.dispose();
    super.dispose();
  }

  void showInSnackBar(String message) {
    ScaffoldMessenger.of(context).showSnackBar(SnackBar(content: Text(message)));
  }

  Future<void> onNewCameraSelected() async {
    photoController = CameraController(widget.xApp.state.cameras[0], ResolutionPreset.medium, enableAudio: true);

    photoController!.addListener(() {
      if (mounted) {
        setState(() {});
      }
      if (photoController!.value.hasError) {
        showInSnackBar('Camera error ${photoController!.value.errorDescription}');
      }
    });

    try {
      await photoController!.initialize();
      await Future.wait(<Future<Object?>>[
        ...!kIsWeb ? <Future<Object?>>[photoController!.getMinExposureOffset().then((double value) => _minAvailableExposureOffset = value), photoController!.getMaxExposureOffset().then((double value) => _maxAvailableExposureOffset = value)] : <Future<Object?>>[],
        photoController!.getMaxZoomLevel().then((double value) => _maxAvailableZoom = value),
        photoController!.getMinZoomLevel().then((double value) => _minAvailableZoom = value),
      ]);
    } on CameraException catch (e) {
      switch (e.code) {
        case 'CameraAccessDenied':
          showInSnackBar('Non hai gli accessi alla telecamera.');
          break;
        case 'CameraAccessDeniedWithoutPrompt':
          // iOS only
          showInSnackBar("Perfavore vai nelle impostazioni dell'app e dai gli accessi alla Telecamera.");
          break;
        case 'CameraAccessRestricted':
          // iOS only
          showInSnackBar('Accesso riservato per la Telecamera');
          break;
        case 'AudioAccessDenied':
          showInSnackBar("Non hai gli accessi per registrare l'audio");
          break;
        case 'AudioAccessDeniedWithoutPrompt':
          // iOS only
          showInSnackBar("Perfavore vai nelle impostazioni dell'app e dai gli accessi all'Audio.");
          break;
        case 'AudioAccessRestricted':
          // iOS only
          showInSnackBar("Accesso riservato per la registrazione Audio");
          break;
        default:
          _showCameraException(e);
          break;
      }
    }

    if (mounted) {
      setState(() {});
    }
  }

  void _handleScaleStart(ScaleStartDetails details) {
    _baseScale = _currentScale;
  }

  Future<void> _handleScaleUpdate(ScaleUpdateDetails details) async {
    if (pointers != 2) {
      return;
    }
    _currentScale = (_baseScale * details.scale).clamp(_minAvailableZoom, _maxAvailableZoom);
    await photoController!.setZoomLevel(_currentScale);
  }

  Widget _cameraPreviewWidget() {
    return Listener(
        onPointerDown: (_) => pointers++,
        onPointerUp: (_) => pointers--,
        child: CameraPreview(photoController!, child: LayoutBuilder(builder: (BuildContext context, BoxConstraints constraints) {
          return GestureDetector(
            behavior: HitTestBehavior.opaque,
            onScaleStart: _handleScaleStart,
            onScaleUpdate: _handleScaleUpdate,
            onTapDown: (TapDownDetails details) => onViewFinderTap(details, constraints),
          );
        })));
  }

  void onViewFinderTap(TapDownDetails details, BoxConstraints constraints) {
    if (photoController == null) {
      return;
    }

    final Offset offset = Offset(
      constraints.maxWidth,
      constraints.maxHeight,
    );
    photoController!.setExposurePoint(offset);
    photoController!.setFocusPoint(offset);
  }

  Future<XFile?> takePicture() async {
    final CameraController? cameraController = photoController!;
    if (cameraController == null || !cameraController.value.isInitialized) {
      return null;
    }
    if (cameraController.value.isTakingPicture) {
      return null;
    }
    try {
      final XFile file = await cameraController.takePicture();
      return file;
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
  }

  void _showCameraException(CameraException e) {
    print(e.description);
    showInSnackBar('Error: ${e.code}\n${e.description}');
  }

  void onTakePictureButtonPressed(T currentDataItem, void Function(void Function()) setState) {
    setState(() {
      waiterMessage = "Sto salvando la Foto...";
      isBusy = true;
    });
    takePicture().then((XFile? file) async {
      if (mounted) {
        this.setState(() {
          imageFile = file;
          videoController?.dispose();
          videoController = null;
        });
        if (file != null) {
          lXMediaUI.add(XMediaItem(Uuid().v4(), payload: await file.readAsBytes()));
          await widget.onTake_Picture(file, currentDataItem, isBusy, setState);
          setState(() {
            waiterMessage = "";
            isBusy = false;
          });
          return file;
        }
      }
    });
  }

  void onVideoRecordButtonPressed(T lav, void Function(void Function()) setState) {
    setState(() {
      waiterMessage = "Sta per cominciare la Registrazione...";
      isBusy = true;
    });
    startVideoRecording(lav, setState).then((_) {
      if (mounted) {
        setState(() {
          waiterMessage = "";
          isBusy = false;
        });
      }
    });
  }

  Widget _captureControlRowWidget(T lav, void Function(void Function()) setState) {
    Color color_BackGroundBTNactions = Color.fromARGB(255, 0, 45, 97);
    return Container(
      width: MediaQuery.of(context).size.width,
      height: 60,
      margin: EdgeInsets.symmetric(vertical: 4),
      child: Row(mainAxisAlignment: MainAxisAlignment.end, children: [
        Container(
            decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
            child: IconButton(
              icon: Icon(Icons.flash_on_outlined),
              color: photoController!.value.flashMode == FlashMode.off
                  ? Colors.white
                  : photoController!.value.isInitialized
                      ? Colors.yellow
                      : Colors.white,
              onPressed: photoController!.value.isInitialized
                  ? photoController!.value.flashMode == FlashMode.off
                      ? () => setFlashMode(FlashMode.torch, setState)
                      : () => setFlashMode(FlashMode.off, setState)
                  : null,
            )),
        SizedBox(width: 4),
        isRecording_Audio
            ? Container()
            : photoController!.value.isRecordingVideo.not()
                ? Container()
                : Container(
                    decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
                    child: IconButton(
                      icon: photoController!.value.isRecordingPaused ? Icon(Icons.play_arrow) : Icon(Icons.pause),
                      color: Colors.white,
                      onPressed: photoController!.value.isInitialized && photoController!.value.isRecordingVideo
                          ? (photoController!.value.isRecordingPaused)
                              ? () => onResumeButtonPressed(setState)
                              : () => onPauseButtonPressed(setState)
                          : null,
                    )),
        SizedBox(width: 4),
        isRecording_Audio
            ? Container()
            : photoController!.value.isRecordingVideo.not()
                ? Container()
                : Container(
                    decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
                    child: IconButton(
                      icon: Icon(Icons.stop),
                      color: Colors.red,
                      onPressed: photoController!.value.isInitialized && photoController!.value.isRecordingVideo ? () => onStopButtonPressed(lav, setState) : null,
                    )),
        // SizedBox(width: 4),
        // photoController!.value.isRecordingVideo || isRecording_Audio
        //     ? Container()
        //     : Container(
        //         decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
        //         child: IconButton(
        //           icon: Icon(Icons.video_camera_back_outlined),
        //           color: outCamera ? Colors.green : Colors.white,
        //           onPressed: photoController!.value.isInitialized && !photoController!.value.isRecordingVideo ? () => cmdViewAnotherCamera() : null,
        //         )),
        SizedBox(width: 4),
        photoController!.value.isRecordingVideo
            ? Container()
            : Container(
                decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
                child: IconButton(
                  icon: Icon(isRecording_Audio.not() ? Icons.mic : Icons.stop),
                  color: isRecording_Audio.not() ? Colors.blue : Colors.red,
                  onPressed: () async => await _startOrStopRecording(lav as XFDataItem),
                )),
        SizedBox(width: 4),
        isRecording_Audio
            ? Container()
            : Container(
                decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
                child: IconButton(
                  icon: Icon(Icons.videocam),
                  color: Colors.white,
                  onPressed: photoController!.value.isInitialized && !photoController!.value.isRecordingVideo ? () => onVideoRecordButtonPressed(lav, setState) : null,
                )),
        SizedBox(width: 4),
        photoController!.value.isRecordingVideo || isRecording_Audio
            ? Container()
            : Container(
                decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(10)),
                child: IconButton(
                  icon: Icon(Icons.camera_alt),
                  color: Colors.white,
                  onPressed: photoController!.value.isInitialized ? () => onTakePictureButtonPressed(lav, setState) : null,
                )),
      ]),

      // Row(mainAxisAlignment: MainAxisAlignment.end, children: [
      //   Row(
      //     mainAxisSize: MainAxisSize.min,
      //     mainAxisAlignment: MainAxisAlignment.end,
      //     children: [
      //       SizedBox(width: isRecording_Audio.not() ? 0 : 10),
      //       isRecording_Audio.not()
      //           ? Container()
      //           : Container(
      //               decoration: BoxDecoration(color: color_BackGroundBTNactions, borderRadius: BorderRadius.circular(20)),
      //               child: IconButton(
      //                 icon: Icon(Icons.stop),
      //                 color: Colors.red,
      //                 onPressed: () async => await _startOrStopRecording(lav as XFDataItem),
      //               )),
      //     ],
      //   ),
      // ]),
    );
  }

  void _getDir() async {
    // appDirectory = await getApplicationDocumentsDirectory();
    // path = "${appDirectory.path}/recording.m4a";
    isLoading = false;
    setState(() {});
  }

  void _initialiseControllers() {
    recorderController = RecorderController()
      ..androidEncoder = AndroidEncoder.aac
      ..androidOutputFormat = AndroidOutputFormat.mpeg4
      ..iosEncoder = IosEncoder.kAudioFormatMPEG4AAC
      ..sampleRate = 44100;
  }

  var idForAudio = Uuid().v4();
  Future _startOrStopRecording(XFDataItem item) async {
    try {
      if (isRecording_Audio) {
        recorderController.reset();
        final path = await recorderController.stop(false);
        if (path != null) {
          isRecordingCompleted = true;
          debugPrint(path);
          debugPrint("Recorded file size: ${File(path).lengthSync()}");
          var xclMedia = XMediaItem(Uuid().v4(), xDocID: item.id, localURI: xFileUI!.path, payload: File(path).readAsBytesSync(), mediaType: XEnums.mediaType.audio.value);
          lXMediaUI.add(xclMedia);

          await widget.onTake_Audio(xclMedia.localURI!, item as T, isBusy, setState);
          var delFile = File("${XLibStorage.filePath}/AUDIO/${xclMedia.id}_.aac");
          delFile.delete();
        }
      } else {
        xFileUI = File("${XLibStorage.filePath}/AUDIO/${idForAudio}_.aac");
        await xFileUI!.create(recursive: true);
        await recorderController.record(path: xFileUI!.path);
      }
    } catch (e) {
      debugPrint(e.toString());
    } finally {
      setState(() {
        isRecording_Audio = !isRecording_Audio;
      });
    }
  }

  cmdViewAnotherCamera() async {
    if (outCamera.not()) {
      var x = await showDialog(
          context: context,
          builder: (context) {
            return XAlertDialog(
                title_Text: "Inserisci URL rtsp:",
                child: Column(mainAxisSize: MainAxisSize.min, children: [
                  Row(children: [Expanded(child: XTextFormField(controller: urlRTSPcontroller, height: null))])
                ]),
                actionsBTNarea_Child: Row(children: [
                  Expanded(
                      child: XBtnbase(
                          margin: EdgeInsets.symmetric(horizontal: 10),
                          label: "Annulla",
                          onPressed: () {
                            urlRTSP = "";
                            Navigator.pop(context, false);
                          },
                          label_Color: Colors.red)),
                  Expanded(
                      child: XBtnbase(
                          margin: EdgeInsets.symmetric(horizontal: 10),
                          label: "Salva",
                          onPressed: () {
                            if (urlRTSPcontroller!.text != urlRTSP) {
                              urlRTSP = urlRTSPcontroller!.text;
                            } else {}
                            outCamera = outCamera.not();
                            Navigator.pop(context, true);
                          },
                          label_Color: Colors.green))
                ]));
          });
      setState(() {
        isBusy = true;
      });
      if (x != null && x) {
        // player.setOption(FijkOption.playerCategory, "mediacodec-all-videos", 1);
        // player = FijkPlayer();
        if (urlRTSP != null) {
          // await player.setDataSource(urlRTSP!, autoPlay: true); //'rtsp://admin:ldmospiti@192.168.1.120:554/live/CAM2'
        }
        // player.isPlayable();
      } else {
        outCamera = false;
      }

      setState(() {
        isBusy = false;
      });
    } else {
      setState(() {
        outCamera = outCamera.not();
      });
    }
  }

  void onStopButtonPressed(T lav, void Function(void Function()) setState) {
    setState(() {
      waiterMessage = "Sto salvando la Registrazione...";
      isBusy = true;
    });
    stopVideoRecording(lav, setState).then((XFile? file) {
      if (mounted) {
        setState(() {});
      }
      if (file != null) {
        videoFile = file;
      }
    });
  }

  Future<void> onPausePreviewButtonPressed(void Function(void Function()) setState) async {
    final CameraController? cameraController = photoController!;

    if (cameraController == null || !cameraController.value.isInitialized) {
      // showInSnackBar('Error: select a camera first.');
      return;
    }
    if (cameraController.value.isPreviewPaused) {
      await cameraController.resumePreview();
    } else {
      await cameraController.pausePreview();
    }

    if (mounted) {
      setState(() {});
    }
  }

  void onPauseButtonPressed(void Function(void Function()) setState) {
    pauseVideoRecording(setState).then((_) {
      if (mounted) {
        setState(() {});
      }
      // showInSnackBar('Video messo in Pausa');
    });
  }

  void onResumeButtonPressed(void Function(void Function()) setState) {
    resumeVideoRecording(setState).then((_) {
      if (mounted) {
        setState(() {});
      }
      // showInSnackBar('Video ripartito');
    });
  }

  Future<void> startVideoRecording(T lav, void Function(void Function()) setState) async {
    if (!photoController!.value.isInitialized) {
      showInSnackBar('Error: Prima seleziona un telecamera.');
      return;
    }

    if (photoController!.value.isRecordingVideo) {
      return;
    }

    try {
      await photoController!.startVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      return;
    }
  }

  Future<Uint8List?> generateVideoThumbnail(String path) async {
    var x = await thumb.VideoThumbnail.thumbnailData(
      video: path,
      imageFormat: thumb.ImageFormat.PNG,
      maxWidth: 128, // specify the width of the thumbnail, let the height auto-scaled to keep the source aspect ratio
      quality: 25,
    );
    return x;
  }

  Future<XFile?> stopVideoRecording(T lav, void Function(void Function()) setState) async {
    if (!photoController!.value.isRecordingVideo) {
      return null;
    }

    try {
      var id = Uuid().v4();
      var y = await photoController!.stopVideoRecording();
      var xFile = File("${XLibStorage.filePath}/VIDEO/${id}_.mp4");
      var bytes = await y.readAsBytes();
      await xFile.writeAsBytes(bytes);
      await xFile.create(recursive: true);
      var minia = await generateVideoThumbnail(xFile.path);

      lXMediaUI.add(XMediaItem(id, payload: bytes, miniatura: minia));
      videoFile = y;
      widget.onTake_Video(videoFile!, lav, isBusy, setState);
      xFile.delete();
      setState(() {
        waiterMessage = "";
        isBusy = false;
      });
      return videoFile;
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
  }

  Future<void> pauseVideoRecording(void Function(void Function()) setState) async {
    if (!photoController!.value.isRecordingVideo) {
      return;
    }

    try {
      await photoController!.pauseVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  Future<void> resumeVideoRecording(void Function(void Function()) setState) async {
    if (!photoController!.value.isRecordingVideo) {
      return;
    }

    try {
      await photoController!.resumeVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  Future<bool> setFlashMode(FlashMode mode, void Function(void Function()) setState) async {
    if (photoController == null) {
      return false;
    }

    try {
      await photoController!.setFlashMode(mode);
      if (photoController!.value.flashMode == FlashMode.off) {
        setState(() {});
        return false;
      } else {
        setState(() {});
        return true;
      }
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  // Widget _thumbnailWidget() {
  //   final VideoPlayerController? localVideoController = videoController;

  //   return Expanded(
  //       child: Align(
  //           alignment: Alignment.centerRight,
  //           child: Row(mainAxisSize: MainAxisSize.min, children: <Widget>[
  //             if (localVideoController == null && imageFile == null)
  //               Container()
  //             else
  //               SizedBox(
  //                   width: 64.0,
  //                   height: 64.0,
  //                   child: (localVideoController == null)
  //                       ? (kIsWeb ? Image.network(imageFile!.path) : Image.file(File(imageFile!.path)))
  //                       : Container(
  //                           decoration: BoxDecoration(border: Border.all(color: Colors.pink)),
  //                           child: Center(
  //                             child: AspectRatio(aspectRatio: localVideoController.value.size != null ? localVideoController.value.aspectRatio : 1.0, child: VideoPlayer(localVideoController)),
  //                           )))
  //           ])));
  // }

  @override
  Widget build(BuildContext context) {
    return (photoController != null && photoController!.value.isInitialized)
        ? Container(
            width: MediaQuery.of(context).size.width - 1,
            height: MediaQuery.of(context).size.height - 1,
            child: Container(
              decoration: BoxDecoration(color: Colors.black, border: Border.all(width: 3.0, color: photoController!.value.isRecordingVideo ? Colors.redAccent : Colors.grey)),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  ///Row delle Preview dei media fatti
                  Row(mainAxisAlignment: lXMediaUI.isNotEmpty ? MainAxisAlignment.start : MainAxisAlignment.center, children: [
                    Expanded(
                        child: SingleChildScrollView(
                      scrollDirection: Axis.horizontal,
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.center,
                        crossAxisAlignment: CrossAxisAlignment.center,
                        children: lXMediaUI.isEmpty
                            ? [Container(height: 80, alignment: Alignment.center, child: Text("...nessun Media registrato...", style: XStyles.xStyTextForTextBase(), textAlign: TextAlign.center))]
                            : lXMediaUI
                                .map(
                                  (e) => e.mediaType == XEnums.mediaType.image.value
                                      ? Container(margin: EdgeInsets.symmetric(horizontal: 5), width: 80, child: Image.memory(e.payload!, width: 80, height: 80))
                                      : e.mediaType == XEnums.mediaType.video.value
                                          ? Container(
                                              margin: EdgeInsets.symmetric(horizontal: 5),
                                              decoration: BoxDecoration(color: Colors.grey[800], image: DecorationImage(image: MemoryImage(e.miniatura!))),
                                              width: 80,
                                              height: 80,
                                              child: Container(
                                                child: Icon(Icons.play_arrow, size: 28),
                                              ))
                                          : Container(
                                              margin: EdgeInsets.symmetric(horizontal: 5),
                                              decoration: BoxDecoration(color: Colors.grey[800]),
                                              width: 80,
                                              height: 80,
                                              child: IconButton(
                                                onPressed: () async {},
                                                icon: Icon(Icons.play_arrow),
                                                color: Colors.white,
                                                splashColor: Colors.transparent,
                                                highlightColor: Colors.transparent,
                                              )),
                                )
                                .toList(),
                      ),
                    ))
                  ]),

                  ///Widget per la registrazione degli audio
                  isRecording_Audio
                      ? AudioWaveforms(
                          size: Size(MediaQuery.of(context).size.width, 200.0),
                          recorderController: recorderController,
                          enableGesture: true,
                          waveStyle: WaveStyle(
                            showDurationLabel: true,
                            spacing: 8.0,
                            showBottom: false,
                            extendWaveform: true,
                            showMiddleLine: false,
                          ),
                        )

                      ///Widget per fare video e foto
                      : Stack(
                          alignment: Alignment.center,
                          children: [
                            _cameraPreviewWidget(),
                            isBusy ? Container(color: Colors.black.withAlpha(90)) : Container(),
                            isBusy ? XStatefulWidgetBase.LoaderIndicator(isBusy, label: waiterMessage) : Container(),
                          ],
                        ),
                  // outCamera == false
                  //     ? _cameraPreviewWidget()
                  //     : FijkView(
                  //         player: player,
                  //),

                  ///Comandi per la gestone dei media
                  _captureControlRowWidget(widget.itemOriginal, setState),
                ],
              ),
            ),
          )
        : XStatefulWidgetBase.LoaderIndicator(true);
  }
}
